#include <stdlib.h>
#include "sauna.h"

/* Internally used globals */
static SDL_Surface *thermometer[2];
static SDL_Surface *stamina[2];
static float temp_delta,temp_delta2;

/* Exported globals */
float temperature;

void initGauges(void) {
    SDL_Surface *tmp;
    /* Load thermometers with colorkey */
	tmp=loadPng("thermo_empty.png");
	if(!tmp) {
		fputs("Couldn't load thermo_empty.png!",stderr);
		exit(-1);
	}
    thermometer[0] = SDL_DisplayFormat(tmp);
    SDL_SetColorKey(thermometer[0],SDL_SRCCOLORKEY|SDL_RLEACCEL,*(Uint16*)thermometer[0]->pixels);
    SDL_FreeSurface(tmp);

	tmp=loadPng("thermo_full.png");
	if(!tmp) {
		fputs("Couldn't load thermo_full.png!",stderr);
		exit(-1);
	}
    thermometer[1] = SDL_DisplayFormat(tmp);
    SDL_SetColorKey(thermometer[1],SDL_SRCCOLORKEY|SDL_RLEACCEL,*(Uint16*)thermometer[1]->pixels);
    SDL_FreeSurface(tmp);

    /* Load beer mugs with alpha */
	tmp=loadPng("stamina_empty.png");
	if(!tmp) {
		fputs("Couldn't load stamina_empty.png!",stderr);
		exit(-1);
	}
    stamina[0]=SDL_DisplayFormatAlpha(tmp);
    SDL_FreeSurface(tmp);

	tmp=loadPng("stamina_full.png");
	if(!tmp) {
		fputs("Couldn't load stamina_full.png!",stderr);
		exit(-1);
	}
    stamina[1]=SDL_DisplayFormatAlpha(tmp);
    SDL_FreeSurface(tmp);

	temperature=60.0;
	temp_delta=0;
}

void deinitGauges(void) {
	SDL_FreeSurface(thermometer[0]);
	SDL_FreeSurface(thermometer[1]);
	SDL_FreeSurface(stamina[0]);
	SDL_FreeSurface(stamina[1]);
}

void animateThermal(float t) {
	temperature-=0.16*t;
	if(temp_delta) {
		temperature+=temp_delta*t;
		temp_delta2+=temp_delta*t;
		if(temp_delta<0) {
			if(temp_delta>temp_delta2) temp_delta=0;
		} else {
			if(temp_delta<temp_delta2) temp_delta=0;
		}
	}
}

void changeTemperature(float d) {
	temp_delta=d;
	temp_delta2=0;
}

#define HOT 37 /* 110C */
#define COLD 252 /* 20C */

void drawThermo(float temp) {
	SDL_Rect src,targ;
	if(temp>110.0) temp=110.0;
	if(temp<20.0) temp=20.0;
	temp=(temp-20.0)/90.0;
	/* Blit empty thermometer */
	src.x=0; src.y=0;
	src.w=thermometer[0]->w;
	src.h=HOT+((COLD-HOT)*(1.0-temp));
	targ.x=screen->w-thermometer[0]->w;
	targ.y=0;
	SDL_BlitSurface(thermometer[0],&src,screen,&targ);
	/* Blit full thermometer */
	targ.y=src.h;
	src.y=src.h;
	src.h=thermometer[1]->h-src.h;
	SDL_BlitSurface(thermometer[1],&src,screen,&targ);
}

void eraseThermo(void) {
	SDL_Rect rect;
	rect.x=screen->w-thermometer[0]->w;
	rect.y=0;
	rect.w=thermometer[0]->w;
	rect.h=thermometer[0]->h;
	SDL_BlitSurface(background,&rect,screen,&rect);
}

/* Alin: 110 */
void drawStamina(float s) {
	SDL_Rect src,targ;
	if(s>1.0) s=1.0;
	else if(s<0.0) s=0.0;
	/* Blit empty stamina gauge */
	src.x=0;
	src.y=0;
	src.w=stamina[0]->w;
	src.h=stamina[0]->h*(1.0-s);
	targ.x=0;
	targ.y=10;
	SDL_BlitSurface(stamina[0],&src,screen,&targ);
	/* Blit full stamina gauge */
	src.y=src.h;
	src.h=stamina[1]->h-src.y;
	targ.y+=src.y;
	SDL_BlitSurface(stamina[1],&src,screen,&targ);
}

void eraseStamina(void) {
	SDL_Rect rect;
	rect.x=0;
	rect.y=10;
	rect.w=stamina[0]->w;
	rect.h=stamina[0]->h;
	SDL_BlitSurface(background,&rect,screen,&rect);
}

