#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include "highscore.h"

HS_List *HS_CreateList(int numRecs, int sizeRec, int (*compar)(const void *a, const void *b))
{
	HS_List *list;

	list = malloc(sizeof(HS_List));
	if(list == NULL) return NULL;

	list->data = malloc((numRecs + 1) * sizeRec);
	if(list->data == NULL)
	{
		free(list);
		return NULL;
	}

	memset(list->data, 0, (numRecs + 1) * sizeRec);

	list->numRecs = numRecs;
	list->sizeRec = sizeRec;
	list->compar = compar;

	return list;
}

void HS_FreeList(HS_List *list)
{
	if(list != NULL && list->data != NULL) free(list->data);
	if(list != NULL) free(list);
}

void *HS_GetRecord(HS_List *list, int num)
{
	void *ptr;

	if(list == NULL || list->data == NULL) return NULL;

	ptr = (void*)((int)list->data + num * list->sizeRec);
	return ptr;
}

void HS_SetRecord(HS_List *list, int num, void *rec)
{
	if(list == NULL || list->data == NULL) return;
	memcpy(HS_GetRecord(list, num), rec, list->sizeRec);
}

void HS_AddRecord(HS_List *list, void *rec)
{
	if(list == NULL || list->data == NULL) return;
	HS_SetRecord(list, list->numRecs, rec);
	qsort(list->data, list->numRecs + 1, list->sizeRec, list->compar);
}

int HS_SaveList(HS_List *list, char *filename)
{
	FILE *f;

	if(list == NULL || list->data == NULL) return 0;

	f = fopen(filename, "wb");
	if(f == NULL)
	{
		return 0;
	}

	fwrite(list, sizeof(HS_List), 1, f);
	fwrite(list->data, list->sizeRec, list->numRecs, f);

	fclose(f);

	return 1;
}

int HS_LoadList(HS_List *list, char *filename)
{
	HS_List l;
	FILE *f;

	if(list == NULL || list->data == NULL) return 0;

	f = fopen(filename, "rb");
	if(f == NULL)
	{
		return 0;
	}

	fread(&l, sizeof(HS_List), 1, f);
	if(list->sizeRec != l.sizeRec || list->numRecs != l.numRecs)
	{
		fclose(f);
		return 0;
	}

	fread(list->data, list->sizeRec, list->numRecs, f);

	fclose(f);

	return 1;
}
