#include "SDL.h"
#include "SDL_mixer.h"

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <math.h>
#include <time.h>

#include "sauna.h"
#include "highscore.h"

SDL_Surface *screen;

/* Exported globals */
SDL_Surface *background;
SDL_Surface *mainmenu;
SDL_Surface *cursor;

/* Internally used globals */
#define SOUND_COUNT 4
typedef enum {LOYLY,SPLASH,BEER,SAUSAGE} Sound;
static char *soundfiles[]={"loyly.wav","splash.wav","beer.wav","sausage.wav"};
static Mix_Chunk *sounds[SOUND_COUNT];

static SDL_Surface *kauha;
static SDL_Surface *drop;
static SDL_Surface *powerup[2];
static SDL_Surface *gameover,*gameovermsg;

/** Main **/
int main(int argc,char *argv[])
{
	int loop;

	if(SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO) != 0)
	{
		fprintf(stderr, "Can't initialize SDL: %s\n", SDL_GetError());
		exit(-1);
	}
	atexit(deInit);

	screen = SDL_SetVideoMode(800, 480, 16, SDL_SWSURFACE | SDL_FULLSCREEN);
	if(screen == NULL)
	{
		fprintf(stderr, "Can't set video mode: %s\n", SDL_GetError());
		SDL_Quit();
		exit(-1);
	}
	
	SDL_WM_SetCaption("Sauna", "Sauna");
	SDL_ShowCursor(SDL_DISABLE);
	chdir("/var/lib/install" PREFIX "/share/" PACKAGE);
	init();

	loop=1;
	while(loop) {
		switch(mainMenu()) {
			case -1: exit(-1);
			case 0:
				if(mainLoop()) {
					loop=0;
				} else {
					if(maxTemperature>getLastScore()) {
						char *name;
						name=inputBox("Kirjoita nimesi",300,20);
						if(!name) loop=0;
						else {
							newHighscore(name);
							if(highscoreMenu()) loop=0;
							free(name);
						}
					}
				}
				break;
			case 1: if(helpMenu()) loop=0; break;
			case 2: if(highscoreMenu()) loop=0; break;
			case 3: loop=0;
		}
	}
	return 0;
}

/** Load graphics **/
void init(void)
{
	SDL_Surface *temp;
	unsigned int i;
	
	srand(time(NULL));

	temp = loadPng("font.png");
	if(temp == NULL)
	{
		fprintf(stderr, "Can't load font.png\n");
		exit(-1);
	}
	font = SDL_DisplayFormat(temp);
    SDL_SetColorKey(font,SDL_SRCCOLORKEY,*(Uint16*)font->pixels);
	SDL_FreeSurface(temp);

	temp = loadPng("cursor.png");
	if(temp == NULL)
	{
		fprintf(stderr, "Can't load cursor.png\n");
		exit(-1);
	}
	cursor = SDL_DisplayFormatAlpha(temp);
	SDL_FreeSurface(temp);

	temp = loadPng("backg.png");
	if(temp == NULL)
	{
		fprintf(stderr, "Can't load backg.png\n");
		exit(-1);
	}
	background = SDL_DisplayFormat(temp);
	SDL_FreeSurface(temp);
	
	temp = loadPng("mainmenu.png");
	if(temp == NULL)
	{
		fprintf(stderr, "Can't load mainmenu.png\n");
		exit(-1);
	}
	mainmenu = SDL_DisplayFormatAlpha(temp);
	SDL_FreeSurface(temp);

	temp = loadPng("kauha.png");
	if(temp == NULL)
	{
		fprintf(stderr, "Can't load kauha.png\n");
		exit(-1);
	}
	kauha = SDL_DisplayFormatAlpha(temp);
	SDL_FreeSurface(temp);

	temp = loadPng("gameover.png");
	if(temp == NULL)
	{
		fprintf(stderr, "Can't load gameover.png\n");
		exit(-1);
	}
	gameover = SDL_DisplayFormatAlpha(temp);
	SDL_FreeSurface(temp);

	temp = loadPng("gameovermsg.png");
	if(temp == NULL)
	{
		fprintf(stderr, "Can't load gameovermsg.png\n");
		exit(-1);
	}
	gameovermsg = SDL_DisplayFormatAlpha(temp);
	SDL_FreeSurface(temp);

    temp = loadPng("drop.png");
    drop = SDL_DisplayFormatAlpha(temp);
    SDL_FreeSurface(temp);

    temp = loadPng("bottle.png");
    powerup[0] = SDL_DisplayFormatAlpha(temp);
    SDL_FreeSurface(temp);

    temp = loadPng("sausage.png");
    powerup[1] = SDL_DisplayFormatAlpha(temp);
    SDL_FreeSurface(temp);

	initHighscores();
	initParticles();
	initGauges();
	
	if(Mix_OpenAudio(MIX_DEFAULT_FREQUENCY,MIX_DEFAULT_FORMAT,MIX_DEFAULT_CHANNELS,256)<0)
	{
		fprintf(stderr,"Cannot open audio: %s\n",SDL_GetError());
	} else
	{
		for(i=0;i<SOUND_COUNT;i++)
		{
			sounds[i]=Mix_LoadWAV(soundfiles[i]);
			if(sounds[i]==NULL)
				fprintf(stderr,"Error ! Couldn't load \"%s\"\n%s\n",soundfiles[i],SDL_GetError());
		}
	}

}

/** Cleanup **/
void deInit(void)
{
	unsigned int i;

	deInitHighscores();

	if(font) SDL_FreeSurface(font);
	if(background) SDL_FreeSurface(background);
	if(kauha) SDL_FreeSurface(kauha);
	if(gameover) SDL_FreeSurface(gameover);
	if(gameovermsg) SDL_FreeSurface(gameovermsg);
    if(drop) SDL_FreeSurface(drop);
    if(powerup[0]) SDL_FreeSurface(powerup[0]);
    if(powerup[1]) SDL_FreeSurface(powerup[1]);

	deinitParticles();
	deinitGauges();
	
	for(i=0;i<SOUND_COUNT;i++)
		Mix_FreeChunk(sounds[i]);

	SDL_Quit();
}

float waterX, waterY, waterVX, waterVY;
float life;

#define GRAVITY gravity
float gravity = 29.0;

#define EMPTY 0
#define THROW 1
#define FLYING 2
int state;

/* stats */
int throws, hits, consecutive, maxConsecutive;
float maxTemperature;
Uint32 gameStart;

#ifndef MAX
#define MAX(a,b) (((a) > (b))?(a):(b))
#endif

#ifndef MIN
#define MIN(a,b) (((a) < (b))?(a):(b))
#endif

#define bbox(ax1, ay1, ax2, ay2, bx1, by1, bx2, by2) ((ax2) > (bx1) && (ay2) > (by1) && (ax1) < (bx2) && (ay1) < (by2))

int segmentIsect(float x11, float y11, float x12, float y12,
				 float x21, float y21, float x22, float y22,
				 float *x, float *y)
{
	float s1x, s1y, s2x, s2y, s, t;

	if(!bbox(MIN(x11, x12), MIN(y11, y12), MAX(x11, x12), MAX(y11, y12),
			 MIN(x21, x22), MIN(y21, y22), MAX(x21, x22), MAX(y21, y22))) return 0;

	s1x = x12 - x11;
	s1y = y12 - y11;

	s2x = x22 - x21;
	s2y = y22 - y21;

	s = (-s1y*(x11-x21) + s1x*(y11-y21))/(-s2x*s1y + s1x*s2y);
	t =  (s2x*(y11-y21) - s2y*(x11-x21))/(-s2x*s1y + s1x*s2y);

	if(s >= 0 && s <= 1 && t >= 0 && t <= 1)
	{
		*x = x11 + t * s1x;
		*y = y11 + t * s1y;
		return 1;
	}

	return 0;
}

static int powerupX=-1,powerupY=-1;

int checkWaterCollision(float x1, float y1, float x2, float y2)
{
	float x, y;
	
	if(segmentIsect(x1, y1, x2, y2, 58.7, 25.1, 68.8, 25.1, &x, &y))
	{
		particleSteam(62.5, 21.6);
		Mix_PlayChannel(-1,sounds[LOYLY],0);
		changeTemperature(10);

		hits++;
		consecutive++;
		if(consecutive > maxConsecutive) maxConsecutive = consecutive;

		return 1;
	}
	
	if(segmentIsect(x1, y1, x2, y2, 58.6, 25.1, 57.6, 40.0, &x, &y))
	{
		particleSplash(x, y);
		Mix_PlayChannel(-1,sounds[SPLASH],0);
		consecutive = 0;
		return 1;
	}

	if(x2 < 1.0 || x2 > 79.0 || y2 > 48.0)
	{
		particleSplash(x2, y2);
		Mix_PlayChannel(-1,sounds[SPLASH],0);
		consecutive = 0;
		return 1;
	}

	if(powerupX!=-1) {
		if(hypot(x2-(powerupX+powerup[0]->h/2)/10.0,y2-(powerupY+powerup[0]->h/2)/10.0)<4.0) {
			particleSplash(x2,y2);

			hits++;
			consecutive++;
			if(consecutive > maxConsecutive) maxConsecutive = consecutive;

			return 2;
		}
	}
	
	return 0;
}

#define ARROW 0
#define KAUHA 1

#define POWERUP_MIN	30.0	/* Minimum time in seconds between powerups */
#define POWERUP_VAR	30.0	/* Powerup interval: POWERUP_MIN+rand*POWERUP_VAR */
int mainLoop(void)
{
	Uint32 startTime;
	float t,gameoverTimer;
	SDL_Event event;
	SDL_Rect rect;
	int mouseX, mouseY;
	float old, new;
	float prevX, prevY;
	int kauhaX, kauhaY;
	float powerupTime,powerupFrame;
	int powerupN=0;
	float x, y;
	int cursorType;
	int quit;
	int frame;
	float stamina;

	stamina = 1.0;

	waterX = 12.5;
	waterY = 50.0;
	state = 0;
	
	old = .5;
	new = .225;

	powerupTime=-(POWERUP_MIN+rand()/(float)RAND_MAX*POWERUP_VAR);
	powerupFrame=0;

	SDL_BlitSurface(background, NULL, screen, NULL);
	
	cursorType = ARROW;
	kauhaX = -1;
	kauhaY = 0;

	throws = 0;
	hits = 0;
	maxTemperature = temperature = 40;
	consecutive = 0;
	maxConsecutive = 0;

	clearParticles();
	
	quit = 0;
	gameoverTimer=0;
	gameStart = startTime = SDL_GetTicks();
	while(!quit)
	{
		t = (SDL_GetTicks() - startTime) / 1000.0;
		if(t < 0.01)
		{
			SDL_Delay(10);
			t += 0.01;
		}
		startTime = SDL_GetTicks();
		
		/* Erase cursor */
		if(kauhaX != -1)
		{
			rect.x = kauhaX;
			rect.y = kauhaY;
			if(cursorType == ARROW)
			{
				rect.w = cursor->w;
				rect.h = cursor->h;
			} else
			{
				rect.w = kauha->w;	
				rect.h = kauha->h;
			}
			SDL_BlitSurface(background, &rect, screen, &rect);
		}

		eraseParticles();
		eraseThermo();
		eraseStamina();

		/* Erase water drop */
		rect.w = 20;
		rect.h = 20;	
		rect.x = (int)(10.0 * waterX) - 10;
		rect.y = (int)(10.0 * waterY) - 10;
		SDL_BlitSurface(background, &rect, screen, &rect);

		/* Erase powerup */
		if(powerupX!=-1) {
			rect.w=powerup[0]->h;
			rect.h=powerup[0]->h;
			rect.x=powerupX;
			rect.y=powerupY;
			SDL_BlitSurface(background,&rect,screen,&rect);
			if(powerupTime<0) powerupX=-1;
		}

		/* Get mouse coordinates and translate */
		SDL_GetMouseState(&mouseX, &mouseY);
		x = mouseX / 10.0;
		y = mouseY / 10.0;

		/* Erase & redraw status */	
		if(state == EMPTY)
		{
			/* Gameover */
			if(gameoverTimer==0) {
				if(stamina <= 0.0) gameoverTimer=3.0;
				if(temperature <= 30) gameoverTimer=3.0;
			}

			if(x < 20.0 && y > 28.0)
			{
				waterX = x;
				waterY = y;
				if(waterX <= 1.0) waterX = 1.0;
				if(waterY >= 47.9) waterY = 47.9;
				
				cursorType = KAUHA;
				kauhaX = mouseX - 20;
				kauhaY = mouseY - 125;
			} else
			{
				cursorType = ARROW;
				kauhaX = mouseX - 5;
				kauhaY = mouseY - 5;
			}
		} else if(state == THROW)
		{
			if(x < 20.0 && y > 28.0)
			{
				waterVX = old * waterVX + new * (x - waterX) / t;
				waterVY = old * waterVY + new * (y - waterY) / t;
				
				waterX = x;
				waterY = y;
				if(waterX <= 1.0) waterX = 1.0;
				if(waterY >= 47.9) waterY = 47.9;
				
				cursorType = KAUHA;
				kauhaX = mouseX - 20;
				kauhaY = mouseY - 125;
			} else
			{
				cursorType = ARROW;
				kauhaX = mouseX - 5;
				kauhaY = mouseY - 5;
				state = EMPTY;
			}
		} else if(state == FLYING)
		{
            SDL_Rect dropsrc;
			prevX = waterX;
			prevY = waterY;
		
			life += t;
			waterVY += GRAVITY * t;
			waterX += waterVX * t;
			waterY += waterVY * t;
			
			switch(checkWaterCollision(prevX, prevY, waterX, waterY))
			{
				case 2:
					stamina+=0.25+(rand()/(float)(RAND_MAX))/2.0;
					if(stamina>1.0) stamina=1.0;
					powerupTime=0.1;
					Mix_PlayChannel(-1,sounds[powerupN?SAUSAGE:BEER],0);
				case 1:
					state = EMPTY;
					waterVX = 0;
					waterVY = 0;
					break;
				default: break;
			}
			
			if((rand()%15)==0) addParticle(waterX,waterY,1-(rand()/(float)RAND_MAX)*2,1-(rand()/(float)RAND_MAX)*2,1.0,0);
			
			rect.w = 20;
			rect.h = 20;	

			rect.x = (int)(10.0 * waterX) - 10;
			rect.y = (int)(10.0 * waterY) - 10;
		
			frame = (int)((life / 10.0) * 25.0);
			if(frame > 24) frame = 24;
            dropsrc.h = drop->h;
            dropsrc.w = drop->h;
            dropsrc.y = 0;
            dropsrc.x = frame*dropsrc.w;
			SDL_BlitSurface(drop, &dropsrc, screen, &rect);

			cursorType = ARROW;
			kauhaX = mouseX - 5;
			kauhaY = mouseY - 5;
		}

		/* Animate particles */
		animateParticles(t);
		/* Animate thermometer */
		animateThermal(t);
		if(temperature > maxTemperature) maxTemperature = temperature;

		/* Animate powerups */
		if(powerupTime<0) {
			powerupTime+=t;
			if(powerupTime>=0) {
				powerupTime=5.0+rand()/(float)RAND_MAX*15.0;
				powerupX=50+rand()/(float)RAND_MAX*550;
				powerupY=50+rand()/(float)RAND_MAX*220;
				if((rand()/(float)RAND_MAX)<0.5) powerupN=0; else powerupN=1;
			}
		} else {
			powerupTime-=t;
			if(powerupTime<=0) {
				powerupTime=-(POWERUP_MIN+rand()/(float)RAND_MAX*POWERUP_VAR);
			}
		}

		/* Draw particles */
		drawParticles();
		/* Draw gauges */
		drawThermo(temperature);
		drawStamina(stamina);
		/* Draw powerups */
		if(powerupTime>0) {
            SDL_Rect powerupsrc;
			rect.x=powerupX;
			rect.y=powerupY;
            powerupsrc.w = powerup[powerupN]->h;
            powerupsrc.h = powerup[powerupN]->h;
            powerupsrc.y = 0;
            powerupsrc.x = (int)(powerupFrame*25.0)%(powerup[powerupN]->w/powerup[powerupN]->h)*powerupsrc.w;
			SDL_BlitSurface(powerup[powerupN],&powerupsrc,screen,&rect);
			powerupFrame+=t;
		}
		/* Draw gameover */
		if(gameoverTimer>0) {
			SDL_Rect rectmsg;
			rect.x=screen->w/2-gameover->w/2;
			rect.y=100;
			rectmsg.x=0;
			rectmsg.w=gameovermsg->w;
			rectmsg.h=gameovermsg->h/2;
			SDL_BlitSurface(gameover,NULL,screen,&rect);
			rect.x=screen->w/2-rectmsg.w/2;
			rect.y=screen->h/2-rectmsg.h/2;
			if(stamina<=0) rectmsg.y=0; else rectmsg.y=rectmsg.h;
			SDL_BlitSurface(gameovermsg,&rectmsg,screen,&rect);
			gameoverTimer-=t;
			if(gameoverTimer<=0) quit=1;
		}
		/* Draw cursor */
		if(kauhaX != -1)
		{
			rect.x = kauhaX;
			rect.y = kauhaY;
			if(cursorType == ARROW)
			{
				rect.w = cursor->w;
				rect.h = cursor->h;
				SDL_BlitSurface(cursor, NULL, screen, &rect);
			} else
			{
				rect.w = kauha->w;	
				rect.h = kauha->h;
				SDL_BlitSurface(kauha, NULL, screen, &rect);
			}
		}

		SDL_Flip(screen);

		while(SDL_PollEvent(&event))
		{
			if(event.type == SDL_QUIT) return 1;
			else if(event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_ESCAPE) quit = 1;
			else if(event.type == SDL_MOUSEBUTTONDOWN && event.button.button == SDL_BUTTON_LEFT)
			{
				if(state == EMPTY && x < 20.0 && y > 28.0 && gameoverTimer==0)
				{
					state = THROW;
				}
			} else if(event.type == SDL_MOUSEBUTTONUP && event.button.button == SDL_BUTTON_LEFT)
			{
				if(state == THROW)
				{
					state = FLYING;
					life = 0;
					throws++;
					stamina -= temperature/800.0;
				}
			}
#ifdef DEBUGMODE
			else if(event.type == SDL_KEYDOWN)
			{
				switch(event.key.keysym.sym)
				{
				case SDLK_UP:
					new += 0.025;
					break;
				case SDLK_DOWN:
					new -= 0.025;
					break;
				case SDLK_LEFT:
					old -= 0.025;
					break;
				case SDLK_RIGHT:
					old += 0.025;
					break;
				case SDLK_KP_PLUS:
					gravity += 0.5;
					break;
				case SDLK_KP_MINUS:
					gravity -= 0.5;
					break;
				case SDLK_PAGEUP:
					changeTemperature(10);
					break;
				case SDLK_PAGEDOWN:
					changeTemperature(-10);
					break;
				default:
					break;
				}
			}
#endif
		}
	}

	gameStart=SDL_GetTicks()-gameStart;
	return 0;
}
