#include <stdlib.h>
#include "sauna.h"

#define NUM_PARTICLES 50

typedef struct {
	float life;
	float life2;
	float x,y;
	float vx,vy;
	SDL_Surface *sprite;
} Particle;

static Particle particles[NUM_PARTICLES];
static SDL_Surface *particle_sprite[2];
static unsigned int currentp;

void initParticles(void) {
	SDL_Surface *temp;
	memset(particles,0,sizeof(Particle)*NUM_PARTICLES);
	currentp=0;
	temp = loadPng("droplet.png");
	if(!temp) {
		fprintf(stderr,"Can't load droplet.png\n");
		exit(-1);
	}
	particle_sprite[0]= SDL_DisplayFormatAlpha(temp);
	SDL_FreeSurface(temp);
	temp=loadPng("fog.png");
	if(!temp) {
		fprintf(stderr,"Can't load .png\n");
	}
	particle_sprite[1]=SDL_DisplayFormatAlpha(temp);
	SDL_FreeSurface(temp);
}

void deinitParticles(void) {
	SDL_FreeSurface(particle_sprite[0]);
	SDL_FreeSurface(particle_sprite[1]);
}

void clearParticles(void) {
	int p;
	for(p=0;p<NUM_PARTICLES;p++) {
		particles[p].life=0;
	}
}

void addParticle(float x,float y,float vx,float vy,float life,int sprite) {
	int i;
	unsigned int oldp=currentp;

	currentp++;
	if(currentp>=NUM_PARTICLES-1) currentp=0;
	i = 0;
	while(particles[currentp].life>0) {
		if(currentp==oldp) return;
		if(currentp>=NUM_PARTICLES-1) currentp=0;
		else currentp++;
	}
	particles[currentp].x=x;
	particles[currentp].y=y;
	particles[currentp].vx=vx;
	particles[currentp].vy=vy;
	particles[currentp].sprite=particle_sprite[sprite];
	particles[currentp].life=life;
	particles[currentp].life2=life;
}

void eraseParticles(void) {
	int r;
	for(r=0;r<NUM_PARTICLES;r++) {
		if(particles[r].life>0) {
			SDL_Rect rect;
			rect.x=particles[r].x*10.0;
			rect.y=particles[r].y*10.0;
			rect.w=particles[r].sprite->h;
			rect.h=particles[r].sprite->h;
			SDL_BlitSurface(background,&rect,screen,&rect);
		}
	}
}

void drawParticles(void) {
	int r;
	for(r=0;r<NUM_PARTICLES;r++) {
		int frame;
		SDL_Rect rect;
		SDL_Rect src;
		if(particles[r].life<=0) continue;
		rect.x=particles[r].x*10.0;
		rect.y=particles[r].y*10.0;
		src.y=0;
		src.w=particles[r].sprite->h;
		src.h=particles[r].sprite->h;
		frame=((particles[r].life2-particles[r].life)/particles[r].life2)*(particles[r].sprite->w/src.h);
		src.x=frame*src.h;
		SDL_BlitSurface(particles[r].sprite,&src,screen,&rect);
	}
}

void animateParticles(float t) {
	int r;
	for(r=0;r<NUM_PARTICLES;r++) {
		if(particles[r].life>0) {
			particles[r].vy+=25.0*t;
			particles[r].x+=particles[r].vx*t;
			particles[r].y+=particles[r].vy*t;
			particles[r].life-=t;
		}
	}
}

void particleSplash(float x,float y) {
	int r;
	for(r=0;r<30;r++) {
		float vx,vy;
		vx=10-((rand()/(float)RAND_MAX)*20.0);
		vy=10-((rand()/(float)RAND_MAX)*20.0);
		addParticle(x,y,vx,vy,3.0,0);
	}
}

void particleSteam(float x,float y) {
	int r;
	for(r=0;r<40;r++) {
		float vx,vy;
		x-=1-((rand()/(float)RAND_MAX)*2.0);
		vx=5-((rand()/(float)RAND_MAX)*10.0);
		vy=-15-((rand()/(float)RAND_MAX)*10.0);
		addParticle(x,y,vx,vy,1.0,1);
	}
}

