#ifndef _SAUNA_H
#define _SAUNA_H

#include "png.h"
#include "SDL.h"

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#include "highscore.h"

extern SDL_Surface *screen;
int mainLoop(void);
void init(void);
void deInit(void);

extern SDL_Surface *background;
extern SDL_Surface *mainmenu;
extern SDL_Surface *cursor;

/* PNG loading  */
SDL_Surface *loadPng(const char *filename);
SDL_Surface *loadPngRW(SDL_RWops * rw);
int savePng(SDL_Surface * surface, const char *filename);

/* Font routines */
extern SDL_Surface *font;
void drawString(char *str, int x, int y);
int gprintf(int x, int y,const char *str, ...);

/* Misc. functions */
extern void surfaceGreyscale(SDL_Surface *surface);
extern char *inputBox(const char *prompt,int width,int maxlen);

/* Menus */
extern int mainMenu(void);
extern int highscoreMenu(void);
extern int helpMenu(void);

/* Particles */
extern void initParticles(void);
extern void deinitParticles(void);
extern void clearParticles(void);
extern void addParticle(float x,float y,float vx,float vy,float life,int sprite);
extern void drawParticles(void);
extern void animateParticles(float t);
extern void particleSplash(float x,float y);
extern void particleSteam(float x,float y);
extern void eraseParticles(void);

/* Gauges */
extern void initGauges(void);
extern void deinitGauges(void);

/* Thermometer */
extern void drawThermo(float temp);
extern void animateThermal(float t);
extern void changeTemperature(float d);
extern void eraseThermo(void);

extern float temperature;

/* Stamina */
extern void eraseStamina(void);
extern void drawStamina(float s);

/* Stats */
extern int throws, hits, consecutive, maxConsecutive;
extern float maxTemperature;
extern Uint32 gameStart;

/* Highscore */
typedef struct score
{
	char name[20];
	int throws, hits;
	float maxTemperature;
	int maxConsecutive;
	int time;
} score;

extern float getLastScore(void);
extern void newHighscore(char *name);
extern void deInitHighscores(void);
extern void initHighscores(void);

extern HS_List *highscores;

#endif
